@@ -3,7 +3,7 @@
 ### Author: Na Li <nali@umn.edu>
 ### Created: 2004/03/12 19:24:20
 ### 
-### $Id: helloR.R,v 1.2 2004/04/15 19:10:30 nali Exp $
+### $Id: helloR.R,v 1.3 2004/05/25 22:18:29 nali Exp $
 ###
 ### Start pvmd if necessary
 
@@ -20,13 +20,16 @@ if (inherits (try (.PVM.config (), silent = TRUE), "try-error")) {
     pvmrun <- TRUE
 }
 
+myname <- paste (Sys.info ()["nodename"], "\n",
+                 Sys.info ()["machine"], "running", Sys.info ()["sysname"],
+                 Sys.info ()["release"])
+
 if (pvmrun) {
     ## Register the master into PVM
     myparent <- .PVM.parent ()
     if (myparent == 0) {
         ## This is master
-        cat ("I am the master", .PVM.mytid (), " from ",
-             Sys.getenv("HOSTNAME"), "\n")
+        cat ("I am the master", .PVM.mytid (), " from ", myname, "\n")
         slaveR <- "helloR.R"
         try (children <- .PVM.spawnR (slaveR, ntask = 1, verbose = TRUE,
                                       flag = "Trace"))
@@ -35,15 +38,14 @@ if (pvmrun) {
         } else {
             bufid <- .PVM.recv ()
             bufinfo <- .PVM.bufinfo (bufid)
-            msg <- .PVM.upkstr (200)
-            cat ("Task ", bufinfo$tid, "said: ", msg, "\n")
+            msg <- .PVM.unserialize ()
+            cat (msg, "\n")
         }
     } else {
         ## This is slave
-        msg <- paste (.PVM.mytid (), "says 'Hello' from",
-                      Sys.getenv ("HOSTNAME"))
+        msg <- paste ("Task", .PVM.mytid (), "said 'Hello' from", myname)
         .PVM.initsend ()
-        .PVM.pkstr (msg)
+        .PVM.serialize (msg)
         .PVM.send (myparent, 1)
     }
 } else {
